'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _en_US = require('../locale-provider/en_US');

var _en_US2 = _interopRequireDefault(_en_US);

var _sprintfJs = require('sprintf-js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var contextTypes = {
  selectSourceData: _react2.default.PropTypes.object,
  antLocale: _react2.default.PropTypes.object
};
//所有表单项props适配器
function propsAdapter(props) {
  var locale = void 0;
  if (this.context.antLocale) {
    locale = this.context.antLocale.FormBuilderCommon;
  } else {
    locale = _en_US2.default.FormBuilderCommon;
  }
  var label = props.label,
      _props$formItemProps = props.formItemProps,
      formItemProps = _props$formItemProps === undefined ? {} : _props$formItemProps,
      value = props.value,
      required = props.required,
      boolean = props.boolean,
      min = props.min,
      max = props.max,
      onlyLetter = props.onlyLetter,
      _props$rules = props.rules,
      rules = _props$rules === undefined ? [] : _props$rules;

  if (!formItemProps.label) {
    formItemProps.label = label;
  }
  min = parseInt(min, 10);
  max = parseInt(max, 10);
  function validateLength() {
    if (min && max) {
      rules.unshift({
        min: min,
        max: max,
        message: (0, _sprintfJs.sprintf)(locale.charactersBetwteen, min, max)
      });
    } else if (min) {
      rules.unshift({
        min: min,
        message: (0, _sprintfJs.sprintf)(locale.charactersMin, min)
      });
    } else if (max) {
      rules.unshift({
        max: max,
        message: (0, _sprintfJs.sprintf)(locale.charactersMax, max)
      });
    }
  }
  //只有text类型有长度限制
  switch (props.type) {
    case "text":
      if (onlyLetter) {
        rules.unshift({
          validator: function validator(rule, value, callback, source, options) {
            var errors = [];
            var pass = new RegExp("^[A-Za-z]*$").test(value);
            if (!pass) {
              errors.push({
                message: locale.charactersOnlyLetter
              });
            }
            callback(errors);
          }
        });
      }
      validateLength();
      break;
    case "password":
      validateLength();
      break;
  }
  if (required) {
    rules.unshift({
      required: true,
      message: (label || props.name) + locale.required
    });
  }
  props.rules = rules;
  props.formItemProps = formItemProps;
  //中间value传递处理
  if (!props.storage) {
    if (this.storage) {
      props.storage = this.storage;
    } else {
      props.storage = {
        value: value
      };
      this.storage = props.storage;
    }
  }
  return props;
}

function formItemComponentDecorator(component) {
  component.prototype.propsAdapter = propsAdapter;
  component.contextTypes = contextTypes;
  return component;
}
exports.default = formItemComponentDecorator;